using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.Json;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using TransmissionRssManager.Core;
using TransmissionRssManager.Services;

namespace TransmissionRssManager.Api.Controllers
{
    [ApiController]
    [Route("api/[controller]")]
    public class ConfigController : ControllerBase
    {
        private readonly ILogger<ConfigController> _logger;
        private readonly IConfigService _configService;

        public ConfigController(
            ILogger<ConfigController> logger,
            IConfigService configService)
        {
            _logger = logger;
            _configService = configService;
        }

        [HttpGet]
        public IActionResult GetConfig()
        {
            var config = _configService.GetConfiguration();
            
            // Create a sanitized config without sensitive information
            var sanitizedConfig = new
            {
                transmission = new
                {
                    host = config.Transmission.Host,
                    port = config.Transmission.Port,
                    useHttps = config.Transmission.UseHttps,
                    hasCredentials = !string.IsNullOrEmpty(config.Transmission.Username),
                    username = config.Transmission.Username
                },
                transmissionInstances = config.TransmissionInstances?.Select(i => new
                {
                    id = i.Key,
                    name = i.Value.Host,
                    host = i.Value.Host,
                    port = i.Value.Port,
                    useHttps = i.Value.UseHttps,
                    hasCredentials = !string.IsNullOrEmpty(i.Value.Username),
                    username = i.Value.Username
                }),
                autoDownloadEnabled = config.AutoDownloadEnabled,
                checkIntervalMinutes = config.CheckIntervalMinutes,
                downloadDirectory = config.DownloadDirectory,
                mediaLibraryPath = config.MediaLibraryPath,
                postProcessing = config.PostProcessing,
                enableDetailedLogging = config.EnableDetailedLogging,
                userPreferences = config.UserPreferences
            };
            
            return Ok(sanitizedConfig);
        }
        
        [HttpGet("defaults")]
        public IActionResult GetDefaultConfig()
        {
            // Return default configuration settings
            var defaultConfig = new
            {
                transmission = new
                {
                    host = "localhost",
                    port = 9091,
                    username = "",
                    useHttps = false
                },
                autoDownloadEnabled = true,
                checkIntervalMinutes = 30,
                downloadDirectory = System.IO.Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.UserProfile), "Downloads"),
                mediaLibraryPath = System.IO.Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.UserProfile), "Media"),
                postProcessing = new
                {
                    enabled = false,
                    extractArchives = true,
                    organizeMedia = true,
                    minimumSeedRatio = 1,
                    mediaExtensions = new[] { ".mp4", ".mkv", ".avi" },
                    autoOrganizeByMediaType = true,
                    renameFiles = false,
                    compressCompletedFiles = false,
                    deleteCompletedAfterDays = 0
                },
                enableDetailedLogging = false,
                userPreferences = new
                {
                    enableDarkMode = false,
                    autoRefreshUIEnabled = true,
                    autoRefreshIntervalSeconds = 30,
                    notificationsEnabled = true,
                    notificationEvents = new[] { "torrent-added", "torrent-completed", "torrent-error" },
                    defaultView = "dashboard",
                    confirmBeforeDelete = true,
                    maxItemsPerPage = 25,
                    dateTimeFormat = "yyyy-MM-dd HH:mm:ss",
                    showCompletedTorrents = true,
                    keepHistoryDays = 30
                }
            };
            
            return Ok(defaultConfig);
        }

        [HttpPut]
        public async Task<IActionResult> UpdateConfig([FromBody] AppConfig config)
        {
            try
            {
                _logger.LogInformation("Received request to update configuration");
                
                if (config == null)
                {
                    _logger.LogError("Received null configuration object");
                    return BadRequest("Configuration cannot be null");
                }
                
                // Log the incoming configuration
                _logger.LogInformation($"Received config with transmission host: {config.Transmission?.Host}, " +
                                      $"autoDownload: {config.AutoDownloadEnabled}");
                
                var currentConfig = _configService.GetConfiguration();
                
                _logger.LogInformation($"Current config has transmission host: {currentConfig.Transmission?.Host}, " +
                                      $"autoDownload: {currentConfig.AutoDownloadEnabled}");
                
                // Make deep copy of current config to start with
                var updatedConfig = JsonSerializer.Deserialize<AppConfig>(
                    JsonSerializer.Serialize(currentConfig),
                    new JsonSerializerOptions { PropertyNameCaseInsensitive = true }
                );
                
                if (updatedConfig == null)
                {
                    _logger.LogError("Failed to create copy of current configuration");
                    return StatusCode(500, "Failed to process configuration update");
                }
                
                // Apply changes from user input
                // Transmission settings
                if (config.Transmission != null)
                {
                    updatedConfig.Transmission.Host = config.Transmission.Host ?? currentConfig.Transmission.Host;
                    updatedConfig.Transmission.Port = config.Transmission.Port;
                    updatedConfig.Transmission.UseHttps = config.Transmission.UseHttps;
                    updatedConfig.Transmission.Username = config.Transmission.Username ?? currentConfig.Transmission.Username;
                    
                    // Only update password if not empty
                    if (!string.IsNullOrEmpty(config.Transmission.Password))
                    {
                        updatedConfig.Transmission.Password = config.Transmission.Password;
                    }
                }
                
                // Core application settings
                updatedConfig.AutoDownloadEnabled = config.AutoDownloadEnabled;
                updatedConfig.CheckIntervalMinutes = config.CheckIntervalMinutes;
                updatedConfig.DownloadDirectory = config.DownloadDirectory ?? currentConfig.DownloadDirectory;
                updatedConfig.MediaLibraryPath = config.MediaLibraryPath ?? currentConfig.MediaLibraryPath;
                updatedConfig.EnableDetailedLogging = config.EnableDetailedLogging;
                
                // Post processing settings
                if (config.PostProcessing != null)
                {
                    updatedConfig.PostProcessing.Enabled = config.PostProcessing.Enabled;
                    updatedConfig.PostProcessing.ExtractArchives = config.PostProcessing.ExtractArchives;
                    updatedConfig.PostProcessing.OrganizeMedia = config.PostProcessing.OrganizeMedia;
                    updatedConfig.PostProcessing.MinimumSeedRatio = config.PostProcessing.MinimumSeedRatio;
                    
                    if (config.PostProcessing.MediaExtensions != null && config.PostProcessing.MediaExtensions.Count > 0)
                    {
                        updatedConfig.PostProcessing.MediaExtensions = config.PostProcessing.MediaExtensions;
                    }
                }
                
                // User preferences
                if (config.UserPreferences != null)
                {
                    updatedConfig.UserPreferences.EnableDarkMode = config.UserPreferences.EnableDarkMode;
                    updatedConfig.UserPreferences.AutoRefreshUIEnabled = config.UserPreferences.AutoRefreshUIEnabled;
                    updatedConfig.UserPreferences.AutoRefreshIntervalSeconds = config.UserPreferences.AutoRefreshIntervalSeconds;
                    updatedConfig.UserPreferences.NotificationsEnabled = config.UserPreferences.NotificationsEnabled;
                }
                
                // Don't lose existing feeds
                // Only update feeds if explicitly provided
                if (config.Feeds != null && config.Feeds.Count > 0)
                {
                    updatedConfig.Feeds = config.Feeds;
                }
                
                // Log the config we're about to save (without sensitive data)
                var sanitizedConfig = new
                {
                    transmission = new
                    {
                        host = updatedConfig.Transmission.Host,
                        port = updatedConfig.Transmission.Port,
                        useHttps = updatedConfig.Transmission.UseHttps,
                        hasUsername = !string.IsNullOrEmpty(updatedConfig.Transmission.Username)
                    },
                    autoDownloadEnabled = updatedConfig.AutoDownloadEnabled,
                    checkIntervalMinutes = updatedConfig.CheckIntervalMinutes,
                    downloadDirectory = updatedConfig.DownloadDirectory,
                    feedCount = updatedConfig.Feeds?.Count ?? 0,
                    postProcessingEnabled = updatedConfig.PostProcessing?.Enabled ?? false,
                    userPreferences = updatedConfig.UserPreferences != null
                };
                _logger.LogInformation("About to save configuration: {@Config}", sanitizedConfig);
                
                await _configService.SaveConfigurationAsync(updatedConfig);
                _logger.LogInformation("Configuration saved successfully");
                
                return Ok(new { success = true, message = "Configuration saved successfully" });
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error saving configuration");
                return StatusCode(500, $"Error saving configuration: {ex.Message}");
            }
        }
        
        [HttpPost("backup")]
        public IActionResult BackupConfig()
        {
            try
            {
                // Get the current config
                var config = _configService.GetConfiguration();
                
                // Serialize to JSON with indentation
                var options = new JsonSerializerOptions { WriteIndented = true };
                var json = JsonSerializer.Serialize(config, options);
                
                // Create a memory stream from the JSON
                var stream = new MemoryStream(Encoding.UTF8.GetBytes(json));
                
                // Set the content disposition and type
                var fileName = $"transmission-rss-config-backup-{DateTime.Now:yyyy-MM-dd}.json";
                return File(stream, "application/json", fileName);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error creating configuration backup");
                return StatusCode(500, "Error creating configuration backup");
            }
        }
        
        [HttpPost("reset")]
        public async Task<IActionResult> ResetConfig()
        {
            try
            {
                // Create a default config
                var defaultConfig = new AppConfig
                {
                    Transmission = new TransmissionConfig
                    {
                        Host = "localhost",
                        Port = 9091,
                        Username = "",
                        Password = "",
                        UseHttps = false
                    },
                    AutoDownloadEnabled = true,
                    CheckIntervalMinutes = 30,
                    DownloadDirectory = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.UserProfile), "Downloads"),
                    MediaLibraryPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.UserProfile), "Media"),
                    PostProcessing = new PostProcessingConfig
                    {
                        Enabled = false,
                        ExtractArchives = true,
                        OrganizeMedia = true,
                        MinimumSeedRatio = 1,
                        MediaExtensions = new List<string> { ".mp4", ".mkv", ".avi", ".mov", ".wmv", ".m4v", ".mpg", ".mpeg", ".flv", ".webm" },
                        AutoOrganizeByMediaType = true,
                        RenameFiles = false,
                        CompressCompletedFiles = false,
                        DeleteCompletedAfterDays = 0
                    },
                    UserPreferences = new TransmissionRssManager.Core.UserPreferences
                    {
                        EnableDarkMode = true,
                        AutoRefreshUIEnabled = true,
                        AutoRefreshIntervalSeconds = 30,
                        NotificationsEnabled = true,
                        NotificationEvents = new List<string> { "torrent-added", "torrent-completed", "torrent-error" },
                        DefaultView = "dashboard",
                        ConfirmBeforeDelete = true,
                        MaxItemsPerPage = 25,
                        DateTimeFormat = "yyyy-MM-dd HH:mm:ss",
                        ShowCompletedTorrents = true,
                        KeepHistoryDays = 30
                    },
                    Feeds = new List<RssFeed>(),
                    EnableDetailedLogging = false
                };
                
                // Save the default config
                await _configService.SaveConfigurationAsync(defaultConfig);
                
                return Ok(new { success = true });
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error resetting configuration");
                return StatusCode(500, "Error resetting configuration");
            }
        }
    }
}