using System;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using TransmissionRssManager.Services;

namespace TransmissionRssManager.Api.Controllers
{
    [ApiController]
    [Route("api/[controller]")]
    public class DashboardController : ControllerBase
    {
        private readonly ILogger<DashboardController> _logger;
        private readonly IMetricsService _metricsService;
        private readonly ILoggingService _loggingService;

        public DashboardController(
            ILogger<DashboardController> logger,
            IMetricsService metricsService,
            ILoggingService loggingService)
        {
            _logger = logger;
            _metricsService = metricsService;
            _loggingService = loggingService;
        }

        [HttpGet("stats")]
        public async Task<IActionResult> GetDashboardStats()
        {
            try
            {
                var stats = await _metricsService.GetDashboardStatsAsync();
                return Ok(stats);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error getting dashboard stats");
                return StatusCode(500, new { error = "Error retrieving dashboard statistics" });
            }
        }

        [HttpGet("history")]
        public async Task<IActionResult> GetDownloadHistory([FromQuery] int days = 30)
        {
            try
            {
                // Get dashboard stats which include necessary information
                var stats = await _metricsService.GetDashboardStatsAsync();
                return Ok(new { 
                    downloadHistory = stats["CompletedTorrents"],
                    period = $"Last {days} days"
                });
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error getting download history");
                return StatusCode(500, new { error = "Error retrieving download history" });
            }
        }

        [HttpGet("categories")]
        public async Task<IActionResult> GetCategoryStats()
        {
            try
            {
                // Create a simplified category breakdown
                var stats = await _metricsService.GetDashboardStatsAsync();
                return Ok(new { 
                    categories = new {
                        active = stats["ActiveDownloads"],
                        seeding = stats["SeedingTorrents"],
                        completed = stats["CompletedTorrents"]
                    }
                });
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error getting category stats");
                return StatusCode(500, new { error = "Error retrieving category statistics" });
            }
        }

        [HttpGet("system")]
        public async Task<IActionResult> GetSystemStatus()
        {
            try
            {
                var status = await _metricsService.GetSystemStatusAsync();
                return Ok(status);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error getting system status");
                return StatusCode(500, new { error = "Error retrieving system status" });
            }
        }

        [HttpGet("disk")]
        public async Task<IActionResult> GetDiskUsage()
        {
            try
            {
                var diskUsageStats = await _metricsService.EstimateDiskUsageAsync();
                return Ok(diskUsageStats);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error getting disk usage");
                _loggingService.Log(
                    LogLevel.Error,
                    $"Error getting disk usage: {ex.Message}",
                    "DashboardController"
                );
                return StatusCode(500, new { error = "Error estimating disk usage" });
            }
        }

        [HttpGet("performance")]
        public async Task<IActionResult> GetPerformanceMetrics()
        {
            try
            {
                // Extract performance metrics from dashboard stats
                var stats = await _metricsService.GetDashboardStatsAsync();
                return Ok(new {
                    downloadSpeed = stats["DownloadSpeed"],
                    uploadSpeed = stats["UploadSpeed"],
                    totalDownloaded = stats["TotalDownloaded"],
                    totalUploaded = stats["TotalUploaded"]
                });
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error getting performance metrics");
                return StatusCode(500, new { error = "Error retrieving performance metrics" });
            }
        }
    }
}