using System;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using TransmissionRssManager.Services;

namespace TransmissionRssManager.Api.Controllers
{
    [ApiController]
    [Route("api/[controller]")]
    public class LogsController : ControllerBase
    {
        private readonly ILogger<LogsController> _logger;
        private readonly ILoggingService _loggingService;

        public LogsController(
            ILogger<LogsController> logger,
            ILoggingService loggingService)
        {
            _logger = logger;
            _loggingService = loggingService;
        }

        [HttpGet]
        public async Task<IActionResult> GetLogs([FromQuery] LogFilterOptions options)
        {
            try
            {
                var logs = await _loggingService.GetLogsAsync(options);
                return Ok(logs);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error getting logs");
                return StatusCode(500, new { error = "Error retrieving logs" });
            }
        }

        [HttpPost("clear")]
        public async Task<IActionResult> ClearLogs([FromQuery] DateTime? olderThan = null)
        {
            try
            {
                await _loggingService.ClearLogsAsync(olderThan);
                return Ok(new { success = true });
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error clearing logs");
                return StatusCode(500, new { error = "Error clearing logs" });
            }
        }

        [HttpGet("export")]
        public async Task<IActionResult> ExportLogs([FromQuery] LogFilterOptions options)
        {
            try
            {
                var exportData = await _loggingService.ExportLogsAsync(options);
                
                // Return as a CSV file download
                return File(exportData, "text/csv", $"transmission-rss-logs-{DateTime.UtcNow:yyyyMMdd-HHmmss}.csv");
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error exporting logs");
                return StatusCode(500, new { error = "Error exporting logs" });
            }
        }

        [HttpPost("message")]
        public IActionResult LogMessage([FromBody] LogMessageRequest request)
        {
            try
            {
                if (string.IsNullOrEmpty(request.Message))
                {
                    return BadRequest(new { error = "Message is required" });
                }

                var logLevel = LogLevel.Information;
                if (!string.IsNullOrEmpty(request.Level) && Enum.TryParse<LogLevel>(request.Level, true, out var parsedLevel))
                {
                    logLevel = parsedLevel;
                }

                _loggingService.Log(logLevel, request.Message, request.Context, request.Properties);
                return Ok(new { success = true });
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error logging message");
                return StatusCode(500, new { error = "Error logging message" });
            }
        }
    }

    public class LogMessageRequest
    {
        public string Message { get; set; }
        public string Level { get; set; } = "Information";
        public string Context { get; set; }
        public System.Collections.Generic.Dictionary<string, string> Properties { get; set; }
    }
}