using Microsoft.AspNetCore.Builder;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.InMemory;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Logging;
using Serilog;
using System;
using System.IO;
using TransmissionRssManager.Core;
using TransmissionRssManager.Data;
using TransmissionRssManager.Data.Repositories;
using TransmissionRssManager.Services;

var builder = WebApplication.CreateBuilder(args);

// Create logs directory for file logging
var logsDirectory = Path.Combine(AppContext.BaseDirectory, "logs");
Directory.CreateDirectory(logsDirectory);

// Configure Serilog
Log.Logger = new LoggerConfiguration()
    .ReadFrom.Configuration(builder.Configuration)
    .Enrich.FromLogContext()
    .WriteTo.Console()
    .WriteTo.File(Path.Combine(logsDirectory, "log-.txt"), rollingInterval: RollingInterval.Day)
    .CreateLogger();

// Use Serilog for logging
builder.Host.UseSerilog();

// Add services to the container
builder.Services.AddControllers();
builder.Services.AddEndpointsApiExplorer();
builder.Services.AddSwaggerGen();

// Configure EF Core with in-memory database for testing
builder.Services.AddDbContext<TorrentManagerContext>(options =>
    options.UseInMemoryDatabase("TransmissionRssManager"));

// Add repositories
builder.Services.AddScoped(typeof(IRepository<>), typeof(Repository<>));
builder.Services.AddScoped<ITorrentRepository, TorrentRepository>();

// Add data migration service
builder.Services.AddScoped<DataMigrationService>();

// Add custom services
builder.Services.AddSingleton<IConfigService, ConfigService>();
builder.Services.AddSingleton<ITransmissionClient, TransmissionClient>();
builder.Services.AddSingleton<IRssFeedManager, RssFeedManager>();
builder.Services.AddSingleton<IPostProcessor, PostProcessor>();
builder.Services.AddSingleton<IMetricsService, MetricsService>();
builder.Services.AddSingleton<ILoggingService, LoggingService>();

// Add scheduler service
builder.Services.AddSingleton<ISchedulerService, SchedulerService>();

// Add background services
builder.Services.AddHostedService<RssFeedBackgroundService>();
builder.Services.AddHostedService<PostProcessingBackgroundService>();

var app = builder.Build();

// Initialize in-memory database
using (var scope = app.Services.CreateScope())
{
    var services = scope.ServiceProvider;
    try
    {
        var context = services.GetRequiredService<TorrentManagerContext>();
        
        // Ensure database is created
        context.Database.EnsureCreated();
        
        // We'll skip the migration service for in-memory DB
        Log.Information("Using in-memory database for testing. No migrations needed.");
    }
    catch (Exception ex)
    {
        var logger = services.GetRequiredService<ILogger<Program>>();
        logger.LogError(ex, "An error occurred while initializing the in-memory database.");
    }
}

// Configure middleware
if (app.Environment.IsDevelopment())
{
    app.UseSwagger();
    app.UseSwaggerUI();
}

// Configure static files to serve index.html as the default file
var options = new DefaultFilesOptions();
options.DefaultFileNames.Clear();
options.DefaultFileNames.Add("index.html");
app.UseDefaultFiles(options);
app.UseStaticFiles();

// Enable directory browsing in development
if (app.Environment.IsDevelopment())
{
    app.UseDirectoryBrowser();
}

app.UseRouting();
app.UseAuthorization();
app.MapControllers();

// Map fallback route for SPA
app.MapFallbackToFile("index.html");

try
{
    await app.RunAsync();
}
catch (Exception ex)
{
    var logger = app.Services.GetRequiredService<ILogger<Program>>();
    logger.LogError(ex, "Application terminated unexpectedly");
}