using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text.Json;
using System.Threading.Tasks;
using Microsoft.Extensions.Logging;
using TransmissionRssManager.Data.Models;
using TransmissionRssManager.Data.Repositories;

namespace TransmissionRssManager.Data
{
    public class DataMigrationService
    {
        private readonly IRepository<RssFeed> _feedRepository;
        private readonly IRepository<RssFeedItem> _feedItemRepository;
        private readonly IRepository<RssFeedRule> _ruleRepository;
        private readonly ITorrentRepository _torrentRepository;
        private readonly IRepository<UserPreference> _preferencesRepository;
        private readonly ILogger<DataMigrationService> _logger;
        
        private readonly string _configDir = Path.Combine(
            Environment.GetFolderPath(Environment.SpecialFolder.UserProfile),
            ".config", "transmission-rss-manager");
            
        private readonly string _dataDir = Path.Combine(
            Environment.GetFolderPath(Environment.SpecialFolder.UserProfile),
            ".local", "share", "transmission-rss-manager");
        
        public DataMigrationService(
            IRepository<RssFeed> feedRepository,
            IRepository<RssFeedItem> feedItemRepository,
            IRepository<RssFeedRule> ruleRepository,
            ITorrentRepository torrentRepository,
            IRepository<UserPreference> preferencesRepository,
            ILogger<DataMigrationService> logger)
        {
            _feedRepository = feedRepository;
            _feedItemRepository = feedItemRepository;
            _ruleRepository = ruleRepository;
            _torrentRepository = torrentRepository;
            _preferencesRepository = preferencesRepository;
            _logger = logger;
        }
        
        public async Task MigrateDataAsync()
        {
            _logger.LogInformation("Beginning data migration from file-based storage to database");
            
            try
            {
                // Migrate configuration preferences
                await MigrateConfigurationAsync();
                
                // Migrate RSS feed definitions and rules
                await MigrateRssFeedsAsync();
                
                // Migrate RSS items and torrent data
                await MigrateRssItemsAsync();
                
                _logger.LogInformation("Data migration completed successfully");
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error during data migration");
                throw;
            }
        }
        
        private async Task MigrateConfigurationAsync()
        {
            var configPath = Path.Combine(_configDir, "config.json");
            if (!File.Exists(configPath))
            {
                _logger.LogWarning("No configuration file found at: {ConfigPath}", configPath);
                return;
            }
            
            try
            {
                var configJson = File.ReadAllText(configPath);
                using var doc = JsonDocument.Parse(configJson);
                var root = doc.RootElement;
                
                // Extract key settings and save as preferences
                if (root.TryGetProperty("TransmissionConfig", out var transmissionConfig))
                {
                    await AddPreferenceAsync("Transmission.Host", 
                        transmissionConfig.GetProperty("Host").GetString());
                        
                    await AddPreferenceAsync("Transmission.Port", 
                        transmissionConfig.GetProperty("Port").GetInt32().ToString());
                        
                    await AddPreferenceAsync("Transmission.Username", 
                        transmissionConfig.GetProperty("Username").GetString());
                        
                    await AddPreferenceAsync("Transmission.Password", 
                        transmissionConfig.GetProperty("Password").GetString());
                        
                    if (transmissionConfig.TryGetProperty("UseHttps", out var useHttps))
                    {
                        await AddPreferenceAsync("Transmission.UseHttps", 
                            useHttps.GetBoolean().ToString().ToLower());
                    }
                }
                
                if (root.TryGetProperty("EnablePostProcessing", out var enablePostProcessing))
                {
                    await AddPreferenceAsync("EnablePostProcessing", 
                        enablePostProcessing.GetBoolean().ToString().ToLower(), 
                        "Enable post-processing of completed torrents");
                }
                
                await _preferencesRepository.SaveChangesAsync();
                _logger.LogInformation("Configuration migration completed");
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error migrating configuration");
                throw;
            }
        }
        
        private async Task MigrateRssFeedsAsync()
        {
            var configPath = Path.Combine(_configDir, "config.json");
            if (!File.Exists(configPath))
            {
                _logger.LogWarning("No configuration file found at: {ConfigPath}", configPath);
                return;
            }
            
            try
            {
                var configJson = File.ReadAllText(configPath);
                using var doc = JsonDocument.Parse(configJson);
                var root = doc.RootElement;
                
                if (root.TryGetProperty("RssFeeds", out var rssFeedsJson))
                {
                    var feedsArray = rssFeedsJson.EnumerateArray();
                    foreach (var feedJson in feedsArray)
                    {
                        var feed = new RssFeed
                        {
                            Name = feedJson.GetProperty("Name").GetString() ?? "Unnamed Feed",
                            Url = feedJson.GetProperty("Url").GetString() ?? "",
                            Enabled = feedJson.TryGetProperty("Enabled", out var enabled) 
                                ? enabled.GetBoolean() 
                                : true,
                            RefreshInterval = feedJson.TryGetProperty("RefreshInterval", out var interval)
                                ? interval.GetInt32()
                                : 15
                        };
                        
                        await _feedRepository.AddAsync(feed);
                        await _feedRepository.SaveChangesAsync();
                        
                        // Process rules if they exist
                        if (feedJson.TryGetProperty("Rules", out var rulesJson))
                        {
                            var rulesArray = rulesJson.EnumerateArray();
                            int priority = 0;
                            
                            foreach (var ruleJson in rulesArray)
                            {
                                var rule = new RssFeedRule
                                {
                                    RssFeedId = feed.Id,
                                    Name = ruleJson.TryGetProperty("Name", out var name)
                                        ? name.GetString() ?? "Unnamed Rule"
                                        : "Unnamed Rule",
                                    IncludePattern = ruleJson.TryGetProperty("IncludePattern", out var include)
                                        ? include.GetString()
                                        : null,
                                    ExcludePattern = ruleJson.TryGetProperty("ExcludePattern", out var exclude)
                                        ? exclude.GetString()
                                        : null,
                                    UseRegex = ruleJson.TryGetProperty("UseRegex", out var regex)
                                        ? regex.GetBoolean()
                                        : false,
                                    Enabled = ruleJson.TryGetProperty("Enabled", out var ruleEnabled)
                                        ? ruleEnabled.GetBoolean()
                                        : true,
                                    SaveToCustomPath = ruleJson.TryGetProperty("SaveToCustomPath", out var customPath)
                                        ? customPath.GetBoolean()
                                        : false,
                                    CustomSavePath = ruleJson.TryGetProperty("CustomSavePath", out var savePath)
                                        ? savePath.GetString()
                                        : null,
                                    EnablePostProcessing = ruleJson.TryGetProperty("EnablePostProcessing", out var postProcess)
                                        ? postProcess.GetBoolean()
                                        : false,
                                    Priority = priority++
                                };
                                
                                await _ruleRepository.AddAsync(rule);
                            }
                            
                            await _ruleRepository.SaveChangesAsync();
                        }
                    }
                    
                    _logger.LogInformation("RSS Feeds migration completed");
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error migrating RSS feeds");
                throw;
            }
        }
        
        private async Task MigrateRssItemsAsync()
        {
            var itemsPath = Path.Combine(_dataDir, "rss-items.json");
            if (!File.Exists(itemsPath))
            {
                _logger.LogWarning("No RSS items file found at: {ItemsPath}", itemsPath);
                return;
            }
            
            try
            {
                var itemsJson = File.ReadAllText(itemsPath);
                var items = JsonSerializer.Deserialize<List<RssFeedItemDTO>>(itemsJson, 
                    new JsonSerializerOptions { PropertyNameCaseInsensitive = true });
                    
                if (items == null) return;
                
                // Get all feeds for lookup
                var feeds = (await _feedRepository.GetAllAsync()).ToDictionary(f => f.Url);
                
                foreach (var itemDto in items)
                {
                    // Skip if we can't match the feed
                    if (!feeds.TryGetValue(itemDto.FeedUrl, out var feed)) continue;
                    
                    var feedItem = new RssFeedItem
                    {
                        RssFeedId = feed.Id,
                        Title = itemDto.Title,
                        Link = itemDto.Link,
                        Description = itemDto.Description,
                        PublishDate = itemDto.PublishDate,
                        IsDownloaded = itemDto.IsDownloaded,
                        DiscoveredAt = itemDto.DiscoveredDate ?? DateTime.UtcNow,
                        DownloadedAt = itemDto.DownloadedDate
                    };
                    
                    await _feedItemRepository.AddAsync(feedItem);
                    await _feedItemRepository.SaveChangesAsync();
                    
                    // If this item has an associated torrent, migrate it too
                    if (itemDto.IsDownloaded && !string.IsNullOrEmpty(itemDto.TorrentHash))
                    {
                        var torrent = new Torrent
                        {
                            Name = itemDto.Title,
                            Hash = itemDto.TorrentHash,
                            TransmissionId = itemDto.TransmissionId,
                            RssFeedItemId = feedItem.Id,
                            AddedOn = itemDto.DownloadedDate ?? feedItem.DiscoveredAt,
                            PostProcessed = itemDto.PostProcessed
                        };
                        
                        await _torrentRepository.AddAsync(torrent);
                        await _torrentRepository.SaveChangesAsync();
                        
                        // Update the feed item with the torrent reference
                        feedItem.TorrentId = torrent.Id;
                        await _feedItemRepository.UpdateAsync(feedItem);
                        await _feedItemRepository.SaveChangesAsync();
                    }
                }
                
                _logger.LogInformation("RSS Items migration completed with {ItemCount} items", items.Count);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error migrating RSS items");
                throw;
            }
        }
        
        private async Task AddPreferenceAsync(string key, string? value, string? description = null)
        {
            var preference = new UserPreference
            {
                Key = key,
                Value = value,
                Description = description,
                Category = key.Contains('.') ? key.Split('.')[0] : "General",
                DataType = int.TryParse(value, out _) ? "int" : 
                          bool.TryParse(value, out _) ? "bool" : "string"
            };
            
            await _preferencesRepository.AddAsync(preference);
        }
        
        // DTO class to help with deserialization
        private class RssFeedItemDTO
        {
            public string Title { get; set; } = string.Empty;
            public string Link { get; set; } = string.Empty;
            public string FeedUrl { get; set; } = string.Empty;
            public string? Description { get; set; }
            public DateTime PublishDate { get; set; }
            public bool IsDownloaded { get; set; }
            public DateTime? DiscoveredDate { get; set; }
            public DateTime? DownloadedDate { get; set; }
            public string? TorrentHash { get; set; }
            public int? TransmissionId { get; set; }
            public bool PostProcessed { get; set; }
        }
    }
}