using System;
using Microsoft.EntityFrameworkCore;
using TransmissionRssManager.Data.Models;

namespace TransmissionRssManager.Data
{
    public class TorrentManagerContext : DbContext
    {
        public TorrentManagerContext(DbContextOptions<TorrentManagerContext> options) : base(options)
        {
        }
        
        public DbSet<Torrent> Torrents { get; set; } = null!;
        public DbSet<RssFeed> RssFeeds { get; set; } = null!;
        public DbSet<RssFeedItem> RssFeedItems { get; set; } = null!;
        public DbSet<RssFeedRule> RssFeedRules { get; set; } = null!;
        public DbSet<UserPreference> UserPreferences { get; set; } = null!;
        public DbSet<SystemLogEntry> SystemLogs { get; set; } = null!;
        
        protected override void OnModelCreating(ModelBuilder modelBuilder)
        {
            // Configure relationships
            
            // RssFeed - RssFeedItem: One-to-Many
            modelBuilder.Entity<RssFeedItem>()
                .HasOne(i => i.RssFeed)
                .WithMany(f => f.Items)
                .HasForeignKey(i => i.RssFeedId)
                .OnDelete(DeleteBehavior.Cascade);
            
            // RssFeed - RssFeedRule: One-to-Many
            modelBuilder.Entity<RssFeedRule>()
                .HasOne(r => r.RssFeed)
                .WithMany(f => f.Rules)
                .HasForeignKey(r => r.RssFeedId)
                .OnDelete(DeleteBehavior.Cascade);
            
            // RssFeedRule - RssFeedItem: One-to-Many (matched items)
            modelBuilder.Entity<RssFeedItem>()
                .HasOne(i => i.MatchedRule)
                .WithMany(r => r.MatchedItems)
                .HasForeignKey(i => i.MatchedRuleId)
                .OnDelete(DeleteBehavior.SetNull);
            
            // RssFeedItem - Torrent: One-to-One
            modelBuilder.Entity<RssFeedItem>()
                .HasOne(i => i.Torrent)
                .WithOne(t => t.RssFeedItem)
                .HasForeignKey<Torrent>(t => t.RssFeedItemId)
                .OnDelete(DeleteBehavior.SetNull);
            
            // Configure indexes for better performance
            modelBuilder.Entity<Torrent>()
                .HasIndex(t => t.Hash)
                .IsUnique();
            
            modelBuilder.Entity<RssFeedItem>()
                .HasIndex(i => new { i.RssFeedId, i.Link })
                .IsUnique();
            
            modelBuilder.Entity<UserPreference>()
                .HasIndex(p => p.Key)
                .IsUnique();
            
            base.OnModelCreating(modelBuilder);
        }
    }
}