#!/bin/bash

# Script to apply database migrations in production
# Usage: ./apply-migrations.sh [connection-string]

set -e # Exit on error

GREEN='\033[0;32m'
YELLOW='\033[1;33m'
RED='\033[0;31m'
NC='\033[0m' # No Color

CONNECTION_STRING=${1:-"Host=localhost;Database=torrentmanager;Username=postgres;Password=postgres"}

# Navigate to the application directory
SCRIPT_DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
APP_DIR="$(dirname "$(dirname "$SCRIPT_DIR")")"
cd "$APP_DIR"

echo -e "${YELLOW}Applying database migrations for Transmission RSS Manager...${NC}"
echo -e "Using connection string: $CONNECTION_STRING"

# Create temp directory for migration
TEMP_DIR=$(mktemp -d)
echo -e "${GREEN}Created temporary directory: $TEMP_DIR${NC}"

# Copy migration files to temp directory
mkdir -p "$TEMP_DIR/Migrations"
cp -r ./Migrations/* "$TEMP_DIR/Migrations/"

# Create a simple program to apply migrations
cat > "$TEMP_DIR/ApplyMigrations.cs" << 'EOF'
using Microsoft.EntityFrameworkCore;
using System;
using TransmissionRssManager.Data;

namespace MigrationUtil
{
    public class Program
    {
        public static void Main(string[] args)
        {
            if (args.Length < 1)
            {
                Console.WriteLine("Usage: dotnet run -- [connection-string]");
                return;
            }

            string connectionString = args[0];
            Console.WriteLine($"Applying migrations using connection string: {connectionString}");

            var optionsBuilder = new DbContextOptionsBuilder<TorrentManagerContext>();
            optionsBuilder.UseNpgsql(connectionString);

            try
            {
                using (var context = new TorrentManagerContext(optionsBuilder.Options))
                {
                    Console.WriteLine("Applying migrations...");
                    context.Database.Migrate();
                    Console.WriteLine("Migrations applied successfully!");
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine($"Error applying migrations: {ex.Message}");
                Console.WriteLine(ex.StackTrace);
                Environment.Exit(1);
            }
        }
    }
}
EOF

# Create project file
cat > "$TEMP_DIR/ApplyMigrations.csproj" << 'EOF'
<Project Sdk="Microsoft.NET.Sdk">
  <PropertyGroup>
    <OutputType>Exe</OutputType>
    <TargetFramework>net7.0</TargetFramework>
  </PropertyGroup>
  
  <ItemGroup>
    <PackageReference Include="Microsoft.EntityFrameworkCore" Version="7.0.16" />
    <PackageReference Include="Microsoft.EntityFrameworkCore.Design" Version="7.0.16">
      <PrivateAssets>all</PrivateAssets>
      <IncludeAssets>runtime; build; native; contentfiles; analyzers; buildtransitive</IncludeAssets>
    </PackageReference>
    <PackageReference Include="Npgsql.EntityFrameworkCore.PostgreSQL" Version="7.0.11" />
  </ItemGroup>
</Project>
EOF

# Copy necessary files from the main project
mkdir -p "$TEMP_DIR/Data/Models"
cp ./src/Data/TorrentManagerContext.cs "$TEMP_DIR/Data/"
cp ./src/Data/Models/*.cs "$TEMP_DIR/Data/Models/"

# Fix namespace issues in TorrentManagerContext.cs
sed -i 's/using TransmissionRssManager.Data.Models;/using TransmissionRssManager.Data.Models;\nusing Microsoft.EntityFrameworkCore.Design;/g' "$TEMP_DIR/Data/TorrentManagerContext.cs"

# Build and run the migration utility
cd "$TEMP_DIR"
echo -e "${GREEN}Building migration utility...${NC}"
dotnet restore
dotnet build -c Release

echo -e "${GREEN}Running migrations...${NC}"
dotnet run -c Release -- "$CONNECTION_STRING"

# Clean up
cd "$APP_DIR"
echo -e "${GREEN}Cleaning up temporary files...${NC}"
rm -rf "$TEMP_DIR"

echo -e "${GREEN}Migration completed successfully!${NC}"