#!/bin/bash

# TransmissionRssManager Installer Script for Linux
# This script installs the TransmissionRssManager application and its dependencies

# Colors for output
GREEN='\033[0;32m'
YELLOW='\033[1;33m'
RED='\033[0;31m'
NC='\033[0m' # No Color

# Error handling
set -e
trap 'echo -e "${RED}An error occurred. Installation failed.${NC}"; exit 1' ERR

# Check if script is run as root
if [ "$EUID" -eq 0 ]; then
  echo -e "${YELLOW}Warning: It's recommended to run this script as a regular user with sudo privileges, not as root.${NC}"
  read -p "Continue anyway? (y/n) " -n 1 -r
  echo
  if [[ ! $REPLY =~ ^[Yy]$ ]]; then
    exit 1
  fi
fi

# Detect Linux distribution
if [ -f /etc/os-release ]; then
    . /etc/os-release
    DISTRO=$ID
else
    echo -e "${RED}Cannot detect Linux distribution. Exiting.${NC}"
    exit 1
fi

echo -e "${GREEN}Installing TransmissionRssManager on $PRETTY_NAME...${NC}"

# Install .NET SDK and runtime
install_dotnet() {
    echo -e "${GREEN}Installing .NET SDK...${NC}"
    
    case $DISTRO in
        ubuntu|debian|linuxmint)
            # Add Microsoft package repository
            wget -O packages-microsoft-prod.deb https://packages.microsoft.com/config/$DISTRO/$VERSION_ID/packages-microsoft-prod.deb
            sudo dpkg -i packages-microsoft-prod.deb
            rm packages-microsoft-prod.deb
            
            # Install .NET SDK
            sudo apt-get update
            sudo apt-get install -y apt-transport-https
            sudo apt-get update
            sudo apt-get install -y dotnet-sdk-7.0
            ;;
        fedora|rhel|centos)
            # Add Microsoft package repository
            sudo rpm -Uvh https://packages.microsoft.com/config/centos/7/packages-microsoft-prod.rpm
            
            # Install .NET SDK
            sudo yum install -y dotnet-sdk-7.0
            ;;
        opensuse*|sles)
            # Install .NET SDK from zypper
            sudo zypper install -y dotnet-sdk-7.0
            ;;
        arch|manjaro)
            # Install .NET SDK from pacman
            sudo pacman -Sy dotnet-sdk aspnet-runtime --noconfirm
            ;;
        *)
            echo -e "${YELLOW}Unsupported distribution for automatic .NET installation.${NC}"
            echo -e "${YELLOW}Please install .NET SDK 7.0 manually from https://dotnet.microsoft.com/download${NC}"
            read -p "Press Enter to continue once .NET SDK is installed..." 
            ;;
    esac
    
    # Verify .NET installation
    dotnet --version
    if [ $? -ne 0 ]; then
        echo -e "${RED}.NET SDK installation failed. Please install .NET SDK 7.0 manually.${NC}"
        exit 1
    fi
}

# Install PostgreSQL
install_postgresql() {
    echo -e "${GREEN}Installing PostgreSQL...${NC}"
    
    case $DISTRO in
        ubuntu|debian|linuxmint)
            sudo apt-get update
            sudo apt-get install -y postgresql postgresql-contrib
            ;;
        fedora)
            sudo dnf install -y postgresql-server postgresql-contrib
            sudo postgresql-setup --initdb --unit postgresql
            sudo systemctl enable postgresql
            sudo systemctl start postgresql
            ;;
        rhel|centos)
            sudo yum install -y postgresql-server postgresql-contrib
            sudo postgresql-setup --initdb --unit postgresql
            sudo systemctl enable postgresql
            sudo systemctl start postgresql
            ;;
        opensuse*|sles)
            sudo zypper install -y postgresql-server postgresql-contrib
            sudo systemctl enable postgresql
            sudo systemctl start postgresql
            ;;
        arch|manjaro)
            sudo pacman -Sy postgresql --noconfirm
            sudo mkdir -p /var/lib/postgres/data
            sudo chown -R postgres:postgres /var/lib/postgres
            sudo su - postgres -c "initdb --locale en_US.UTF-8 -D '/var/lib/postgres/data'"
            sudo systemctl enable postgresql
            sudo systemctl start postgresql
            ;;
        *)
            echo -e "${YELLOW}Unsupported distribution for automatic PostgreSQL installation.${NC}"
            echo -e "${YELLOW}Please install PostgreSQL manually.${NC}"
            read -p "Press Enter to continue once PostgreSQL is installed..." 
            ;;
    esac
    
    # Verify PostgreSQL installation
    if ! command -v psql &> /dev/null; then
        echo -e "${RED}PostgreSQL installation failed. Please install PostgreSQL manually.${NC}"
        read -p "Do you want to continue with the installation without PostgreSQL? (y/n) " -n 1 -r
        echo
        if [[ ! $REPLY =~ ^[Yy]$ ]]; then
            exit 1
        fi
        SKIP_DB_SETUP=true
    fi
}

# Setup PostgreSQL database
setup_database() {
    if [ "$SKIP_DB_SETUP" = true ]; then
        echo -e "${YELLOW}Skipping database setup.${NC}"
        return
    fi
    
    echo -e "${GREEN}Setting up PostgreSQL database...${NC}"
    
    DB_NAME="torrentmanager"
    DB_USER="torrentmanager"
    
    # Generate a random password
    DB_PASSWORD=$(tr -dc 'A-Za-z0-9' < /dev/urandom | head -c 16)
    
    # Create user and database
    sudo -u postgres psql -c "CREATE USER $DB_USER WITH PASSWORD '$DB_PASSWORD';"
    sudo -u postgres psql -c "CREATE DATABASE $DB_NAME OWNER $DB_USER;"
    sudo -u postgres psql -c "GRANT ALL PRIVILEGES ON DATABASE $DB_NAME TO $DB_USER;"
    
    echo -e "${GREEN}Database setup completed.${NC}"
    
    # Save database connection information to configuration
    DATABASE_CONFIG="$HOME/.config/transmission-rss-manager/database.json"
    echo '{
  "ConnectionStrings": {
    "DefaultConnection": "Host=localhost;Database='$DB_NAME';Username='$DB_USER';Password='$DB_PASSWORD'"
  }
}' > "$DATABASE_CONFIG"
    
    echo -e "${GREEN}Database configuration saved to $DATABASE_CONFIG${NC}"
}

# Install dependencies
install_dependencies() {
    echo -e "${GREEN}Installing dependencies...${NC}"
    
    case $DISTRO in
        ubuntu|debian|linuxmint)
            sudo apt-get update
            sudo apt-get install -y unzip p7zip-full unrar-free libssl-dev zlib1g-dev libicu-dev build-essential
            ;;
        fedora|rhel|centos)
            sudo yum install -y unzip p7zip unrar openssl-devel zlib-devel libicu-devel gcc-c++ make
            ;;
        opensuse*|sles)
            sudo zypper install -y unzip p7zip unrar libopenssl-devel zlib-devel libicu-devel gcc-c++ make
            ;;
        arch|manjaro)
            sudo pacman -Sy unzip p7zip unrar openssl zlib icu gcc make --noconfirm
            ;;
        *)
            echo -e "${YELLOW}Unsupported distribution for automatic dependency installation.${NC}"
            echo -e "${YELLOW}Please make sure the following are installed: unzip, p7zip, unrar, ssl, zlib, icu, gcc/g++ and make.${NC}"
            ;;
    esac
    
    # Install Entity Framework Core CLI tools if needed (version 7.x)
    if ! command -v dotnet-ef &> /dev/null; then
        echo -e "${GREEN}Installing Entity Framework Core tools compatible with .NET 7...${NC}"
        dotnet tool install --global dotnet-ef --version 7.0.15
    fi
}

# Check if .NET is already installed
if command -v dotnet >/dev/null 2>&1; then
    dotnet_version=$(dotnet --version)
    echo -e "${GREEN}.NET SDK version $dotnet_version is already installed.${NC}"
else
    install_dotnet
fi

# Check if PostgreSQL is already installed
if command -v psql >/dev/null 2>&1; then
    pg_version=$(psql --version | grep -o 'PostgreSQL [0-9]\+\.[0-9]\+')
    echo -e "${GREEN}$pg_version is already installed.${NC}"
    
    # Check if database exists
    EXISTING_DB=$(sudo -u postgres psql -tAc "SELECT 1 FROM pg_database WHERE datname='torrentmanager'")
    if [ "$EXISTING_DB" = "1" ]; then
        echo -e "${GREEN}Database 'torrentmanager' already exists.${NC}"
        read -p "Do you want to use the existing database? (y/n) " -n 1 -r
        echo
        if [[ ! $REPLY =~ ^[Yy]$ ]]; then
            setup_database
        else
            echo -e "${GREEN}Using existing database.${NC}"
            # TODO: Update connection string with existing credentials if needed
        fi
    else
        setup_database
    fi
else
    install_postgresql
    setup_database
fi

# Install dependencies
install_dependencies

# Create installation directory
INSTALL_DIR="$HOME/.local/share/transmission-rss-manager"
mkdir -p "$INSTALL_DIR"

# Clone or download the application
echo -e "${GREEN}Downloading TransmissionRssManager...${NC}"
if [ -d "/opt/develop/transmission-rss-manager/TransmissionRssManager" ]; then
    # We're running from the development directory
    cp -r /opt/develop/transmission-rss-manager/TransmissionRssManager/* "$INSTALL_DIR/"
else
    # Download and extract release
    wget -O transmission-rss-manager.zip https://github.com/yourusername/transmission-rss-manager/releases/latest/download/transmission-rss-manager.zip
    unzip transmission-rss-manager.zip -d "$INSTALL_DIR"
    rm transmission-rss-manager.zip
fi

# Install required NuGet packages (with versions compatible with .NET 7)
echo -e "${GREEN}Installing required NuGet packages...${NC}"
cd "$INSTALL_DIR"
dotnet add package Microsoft.AspNetCore.OpenApi --version 7.0.13
dotnet add package Swashbuckle.AspNetCore --version 6.5.0
dotnet add package System.ServiceModel.Syndication --version 7.0.0
dotnet add package Microsoft.EntityFrameworkCore --version 7.0.17
dotnet add package Microsoft.EntityFrameworkCore.Design --version 7.0.17
dotnet add package Npgsql.EntityFrameworkCore.PostgreSQL --version 7.0.11

# Build the application
echo -e "${GREEN}Building TransmissionRssManager...${NC}"
dotnet build -c Release

# Copy database configuration
CONFIG_DIR="$HOME/.config/transmission-rss-manager"
mkdir -p "$CONFIG_DIR"

# Copy database.json to appsettings.json if it exists
if [ -f "$CONFIG_DIR/database.json" ]; then
    # Merge database.json with any existing appsettings.json
    if [ -f "$INSTALL_DIR/appsettings.json" ]; then
        cp "$INSTALL_DIR/appsettings.json" "$CONFIG_DIR/appsettings.json.bak"
        echo -e "${GREEN}Backed up existing appsettings.json to appsettings.json.bak${NC}"
        # Extract connection string from database.json
        CONNECTION_STRING=$(grep -o '"DefaultConnection": "[^"]*"' "$CONFIG_DIR/database.json")
        # Update appsettings.json with connection string
        if grep -q '"ConnectionStrings"' "$INSTALL_DIR/appsettings.json"; then
            # Update existing ConnectionStrings section
            sed -i 's/"DefaultConnection": "[^"]*"/'$CONNECTION_STRING'/' "$INSTALL_DIR/appsettings.json"
        else
            # Create ConnectionStrings section
            CONNECTION_SECTION=$(grep -A 3 '"ConnectionStrings"' "$CONFIG_DIR/database.json")
            # Insert ConnectionStrings section before the last }
            sed -i '$ s/}/,\n  '$CONNECTION_SECTION'\n}/' "$INSTALL_DIR/appsettings.json"
        fi
    else
        # Create new appsettings.json
        cp "$CONFIG_DIR/database.json" "$INSTALL_DIR/appsettings.json"
    fi
fi

# Run initial database migration if needed
echo -e "${GREEN}Setting up database schema...${NC}"
cd "$INSTALL_DIR"
dotnet ef database update

# Create desktop entry
DESKTOP_FILE="$HOME/.local/share/applications/transmission-rss-manager.desktop"
echo "[Desktop Entry]
Name=Transmission RSS Manager
Comment=RSS Feed Manager for Transmission BitTorrent Client
Exec=dotnet $INSTALL_DIR/bin/Release/net7.0/TransmissionRssManager.dll
Icon=transmission
Terminal=false
Type=Application
Categories=Network;P2P;" > "$DESKTOP_FILE"

# Create systemd service for user
SERVICE_DIR="$HOME/.config/systemd/user"
mkdir -p "$SERVICE_DIR"

echo "[Unit]
Description=Transmission RSS Manager
After=network.target postgresql.service

[Service]
ExecStart=dotnet $INSTALL_DIR/bin/Release/net7.0/TransmissionRssManager.dll
Restart=on-failure
RestartSec=10
SyslogIdentifier=transmission-rss-manager

[Install]
WantedBy=default.target" > "$SERVICE_DIR/transmission-rss-manager.service"

# Reload systemd
systemctl --user daemon-reload

# Create launcher script
LAUNCHER="$HOME/.local/bin/transmission-rss-manager"
mkdir -p "$HOME/.local/bin"

echo "#!/bin/bash
dotnet $INSTALL_DIR/bin/Release/net7.0/TransmissionRssManager.dll" > "$LAUNCHER"
chmod +x "$LAUNCHER"

echo -e "${GREEN}Installation completed!${NC}"
echo -e "${GREEN}You can run TransmissionRssManager in these ways:${NC}"
echo -e "  * Command: ${YELLOW}transmission-rss-manager${NC}"
echo -e "  * Service: ${YELLOW}systemctl --user start transmission-rss-manager${NC}"
echo -e "  * Enable service on startup: ${YELLOW}systemctl --user enable transmission-rss-manager${NC}"
echo -e "  * Web interface will be available at: ${YELLOW}http://localhost:5000${NC}"

# Start the application
read -p "Do you want to start the application now? (y/n) " -n 1 -r
echo
if [[ $REPLY =~ ^[Yy]$ ]]; then
    systemctl --user start transmission-rss-manager
    echo -e "${GREEN}TransmissionRssManager service started.${NC}"
    echo -e "${GREEN}Open http://localhost:5000 in your browser.${NC}"
else
    echo -e "${YELLOW}You can start the application later using: systemctl --user start transmission-rss-manager${NC}"
fi