using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Logging;
using TransmissionRssManager.Core;

namespace TransmissionRssManager.Services
{
    public class PostProcessor : IPostProcessor
    {
        private readonly ILogger<PostProcessor> _logger;
        private readonly IConfigService _configService;
        private readonly ITransmissionClient _transmissionClient;
        private readonly List<TorrentInfo> _completedTorrents = new List<TorrentInfo>();

        public PostProcessor(
            ILogger<PostProcessor> logger,
            IConfigService configService,
            ITransmissionClient transmissionClient)
        {
            _logger = logger;
            _configService = configService;
            _transmissionClient = transmissionClient;
        }

        public async Task ProcessCompletedDownloadsAsync(CancellationToken cancellationToken)
        {
            _logger.LogInformation("Checking for completed downloads");
            
            var config = _configService.GetConfiguration();
            if (!config.PostProcessing.Enabled)
            {
                _logger.LogInformation("Post-processing is disabled");
                return;
            }
            
            try
            {
                var torrents = await _transmissionClient.GetTorrentsAsync();
                var completedTorrents = torrents.Where(t => t.IsFinished && !_completedTorrents.Any(c => c.Id == t.Id)).ToList();
                
                _logger.LogInformation($"Found {completedTorrents.Count} newly completed torrents");
                
                foreach (var torrent in completedTorrents)
                {
                    if (cancellationToken.IsCancellationRequested)
                        break;
                        
                    await ProcessTorrentAsync(torrent);
                    _completedTorrents.Add(torrent);
                }
                
                // Clean up the list of completed torrents to avoid memory leaks
                if (_completedTorrents.Count > 1000)
                {
                    _completedTorrents.RemoveRange(0, _completedTorrents.Count - 1000);
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error processing completed downloads");
            }
        }

        public async Task ProcessTorrentAsync(TorrentInfo torrent)
        {
            _logger.LogInformation($"Processing completed torrent: {torrent.Name}");
            
            var config = _configService.GetConfiguration();
            var processingConfig = config.PostProcessing;
            
            if (!Directory.Exists(torrent.DownloadDir))
            {
                _logger.LogWarning($"Download directory does not exist: {torrent.DownloadDir}");
                return;
            }
            
            try
            {
                // Extract archives if enabled
                if (processingConfig.ExtractArchives)
                {
                    await ExtractArchivesAsync(torrent.DownloadDir);
                }
                
                // Organize media if enabled
                if (processingConfig.OrganizeMedia && !string.IsNullOrEmpty(config.MediaLibraryPath))
                {
                    await OrganizeMediaAsync(torrent.DownloadDir, config.MediaLibraryPath, processingConfig);
                }
                
                _logger.LogInformation($"Completed processing torrent: {torrent.Name}");
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, $"Error processing torrent: {torrent.Name}");
            }
        }
        
        private async Task ExtractArchivesAsync(string directory)
        {
            _logger.LogInformation($"Extracting archives in {directory}");
            
            var archiveExtensions = new[] { ".rar", ".zip", ".7z", ".tar", ".gz" };
            var archiveFiles = Directory.GetFiles(directory, "*.*", SearchOption.AllDirectories)
                .Where(f => archiveExtensions.Contains(Path.GetExtension(f).ToLowerInvariant()))
                .ToList();
                
            foreach (var archiveFile in archiveFiles)
            {
                try
                {
                    _logger.LogInformation($"Extracting archive: {archiveFile}");
                    
                    var dirName = Path.GetDirectoryName(archiveFile);
                    var fileName = Path.GetFileNameWithoutExtension(archiveFile);
                    
                    if (dirName == null)
                    {
                        _logger.LogWarning($"Could not get directory name for archive: {archiveFile}");
                        continue;
                    }
                    
                    var extractDir = Path.Combine(dirName, fileName);
                        
                    if (!Directory.Exists(extractDir))
                    {
                        Directory.CreateDirectory(extractDir);
                    }
                    
                    await Task.Run(() => ExtractWithSharpCompress(archiveFile, extractDir));
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, $"Error extracting archive: {archiveFile}");
                }
            }
        }
        
        private void ExtractWithSharpCompress(string archiveFile, string extractDir)
        {
            // In a real implementation, this would use SharpCompress to extract files
            _logger.LogInformation($"Would extract {archiveFile} to {extractDir}");
            // For testing, we'll create a dummy file to simulate extraction
            File.WriteAllText(
                Path.Combine(extractDir, "extracted.txt"), 
                $"Extracted from {archiveFile} at {DateTime.Now}"
            );
        }
        
        private async Task OrganizeMediaAsync(string sourceDir, string targetDir, PostProcessingConfig config)
        {
            _logger.LogInformation($"Organizing media from {sourceDir} to {targetDir}");
            
            if (!Directory.Exists(targetDir))
            {
                Directory.CreateDirectory(targetDir);
            }
            
            var mediaFiles = Directory.GetFiles(sourceDir, "*.*", SearchOption.AllDirectories)
                .Where(f => config.MediaExtensions.Contains(Path.GetExtension(f).ToLowerInvariant()))
                .ToList();
                
            foreach (var mediaFile in mediaFiles)
            {
                try
                {
                    _logger.LogInformation($"Processing media file: {mediaFile}");
                    
                    string destFolder = targetDir;
                    
                    // Organize by media type if enabled
                    if (config.AutoOrganizeByMediaType)
                    {
                        string mediaType = DetermineMediaType(mediaFile);
                        destFolder = Path.Combine(targetDir, mediaType);
                        
                        if (!Directory.Exists(destFolder))
                        {
                            Directory.CreateDirectory(destFolder);
                        }
                    }
                    
                    string destFile = Path.Combine(destFolder, Path.GetFileName(mediaFile));
                    
                    // Rename file if needed
                    if (config.RenameFiles)
                    {
                        string newFileName = CleanFileName(Path.GetFileName(mediaFile));
                        destFile = Path.Combine(destFolder, newFileName);
                    }
                    
                    // Copy file (in real implementation we might move instead)
                    await Task.Run(() => File.Copy(mediaFile, destFile, true));
                    
                    _logger.LogInformation($"Copied {mediaFile} to {destFile}");
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, $"Error processing media file: {mediaFile}");
                }
            }
        }
        
        private string DetermineMediaType(string filePath)
        {
            // In a real implementation, this would analyze the file to determine its type
            // For now, just return a simple category based on extension
            
            string ext = Path.GetExtension(filePath).ToLowerInvariant();
            
            if (new[] { ".mp4", ".mkv", ".avi", ".mov" }.Contains(ext))
            {
                return "Videos";
            }
            else if (new[] { ".mp3", ".flac", ".wav", ".aac" }.Contains(ext))
            {
                return "Music";
            }
            else if (new[] { ".jpg", ".png", ".gif", ".bmp" }.Contains(ext))
            {
                return "Images";
            }
            else
            {
                return "Other";
            }
        }
        
        private string CleanFileName(string fileName)
        {
            // Replace invalid characters and clean up the filename
            string invalidChars = new string(Path.GetInvalidFileNameChars());
            string invalidReStr = string.Format(@"[{0}]", Regex.Escape(invalidChars));
            
            // Remove scene tags, dots, underscores, etc.
            string cleanName = fileName
                .Replace(".", " ")
                .Replace("_", " ");
                
            // Replace invalid characters
            cleanName = Regex.Replace(cleanName, invalidReStr, "");
            
            // Remove extra spaces
            cleanName = Regex.Replace(cleanName, @"\s+", " ").Trim();
            
            // Add original extension if it was removed
            string originalExt = Path.GetExtension(fileName);
            if (!cleanName.EndsWith(originalExt, StringComparison.OrdinalIgnoreCase))
            {
                cleanName += originalExt;
            }
            
            return cleanName;
        }
    }
    
    public class PostProcessingBackgroundService : BackgroundService
    {
        private readonly ILogger<PostProcessingBackgroundService> _logger;
        private readonly IServiceProvider _serviceProvider;
        
        public PostProcessingBackgroundService(
            ILogger<PostProcessingBackgroundService> logger,
            IServiceProvider serviceProvider)
        {
            _logger = logger;
            _serviceProvider = serviceProvider;
        }
        
        protected override async Task ExecuteAsync(CancellationToken stoppingToken)
        {
            _logger.LogInformation("Post-processing background service started");
            
            while (!stoppingToken.IsCancellationRequested)
            {
                try
                {
                    using (var scope = _serviceProvider.CreateScope())
                    {
                        var postProcessor = scope.ServiceProvider.GetRequiredService<IPostProcessor>();
                        var configService = scope.ServiceProvider.GetRequiredService<IConfigService>();
                        
                        await postProcessor.ProcessCompletedDownloadsAsync(stoppingToken);
                        
                        // Check every minute for completed downloads
                        await Task.Delay(TimeSpan.FromMinutes(1), stoppingToken);
                    }
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Error in post-processing background service");
                    await Task.Delay(TimeSpan.FromMinutes(5), stoppingToken);
                }
            }
            
            _logger.LogInformation("Post-processing background service stopped");
        }
    }
}