using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net.Http;
using System.ServiceModel.Syndication;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;
using System.Xml;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Logging;
using TransmissionRssManager.Core;

namespace TransmissionRssManager.Services
{
    public class RssFeedManager : IRssFeedManager
    {
        private readonly ILogger<RssFeedManager> _logger;
        private readonly IConfigService _configService;
        private readonly ITransmissionClient _transmissionClient;
        private List<RssFeed> _feeds = new List<RssFeed>();
        private List<RssFeedItem> _feedItems = new List<RssFeedItem>();
        private readonly HttpClient _httpClient;

        public RssFeedManager(
            ILogger<RssFeedManager> logger,
            IConfigService configService,
            ITransmissionClient transmissionClient)
        {
            _logger = logger;
            _configService = configService;
            _transmissionClient = transmissionClient;
            _httpClient = new HttpClient();
            
            // Load feeds from config
            var config = _configService.GetConfiguration();
            _feeds = config.Feeds;
        }
        
        public async Task<List<RssFeedItem>> GetAllItemsAsync()
        {
            return _feedItems;
        }
        
        public async Task<List<RssFeedItem>> GetMatchedItemsAsync()
        {
            return _feedItems.Where(item => item.IsMatched).ToList();
        }
        
        public async Task<List<RssFeed>> GetFeedsAsync()
        {
            return _feeds;
        }
        
        public async Task AddFeedAsync(RssFeed feed)
        {
            feed.Id = Guid.NewGuid().ToString();
            _feeds.Add(feed);
            await SaveFeedsToConfigAsync();
        }
        
        public async Task RemoveFeedAsync(string feedId)
        {
            _feeds.RemoveAll(f => f.Id == feedId);
            _feedItems.RemoveAll(i => i.FeedId == feedId);
            await SaveFeedsToConfigAsync();
        }
        
        public async Task UpdateFeedAsync(RssFeed feed)
        {
            var existingFeed = _feeds.FirstOrDefault(f => f.Id == feed.Id);
            if (existingFeed != null)
            {
                int index = _feeds.IndexOf(existingFeed);
                _feeds[index] = feed;
                await SaveFeedsToConfigAsync();
            }
        }
        
        private async Task SaveFeedsToConfigAsync()
        {
            var config = _configService.GetConfiguration();
            config.Feeds = _feeds;
            await _configService.SaveConfigurationAsync(config);
        }
        
        public async Task RefreshFeedsAsync(CancellationToken cancellationToken)
        {
            _logger.LogInformation("Refreshing RSS feeds");
            
            foreach (var feed in _feeds.Where(f => f.Enabled))
            {
                if (cancellationToken.IsCancellationRequested)
                    break;
                
                try
                {
                    await RefreshFeedAsync(feed.Id, cancellationToken);
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, $"Error refreshing feed {feed.Name}");
                }
            }
        }
        
        public async Task RefreshFeedAsync(string feedId, CancellationToken cancellationToken)
        {
            var feed = _feeds.FirstOrDefault(f => f.Id == feedId);
            if (feed == null)
                return;
            
            try
            {
                _logger.LogInformation($"Refreshing feed: {feed.Name}");
                
                var feedItems = await FetchFeedItemsAsync(feed.Url);
                foreach (var item in feedItems)
                {
                    // Add only if we don't already have this item
                    if (!_feedItems.Any(i => i.Link == item.Link && i.FeedId == feed.Id))
                    {
                        item.FeedId = feed.Id;
                        _feedItems.Add(item);
                        
                        // Apply rules
                        ApplyRulesToItem(feed, item);
                        
                        // Download if matched and auto-download is enabled
                        if (item.IsMatched && feed.AutoDownload)
                        {
                            await DownloadMatchedItemAsync(item);
                        }
                    }
                }
                
                // Update last checked time
                feed.LastChecked = DateTime.UtcNow;
                feed.ErrorCount = 0;
                feed.LastErrorMessage = string.Empty;
                
                // Cleanup old items
                CleanupOldItems(feed);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, $"Error refreshing feed {feed.Name}: {ex.Message}");
                feed.ErrorCount++;
                feed.LastError = DateTime.UtcNow;
                feed.LastErrorMessage = ex.Message;
            }
        }
        
        private async Task<List<RssFeedItem>> FetchFeedItemsAsync(string url)
        {
            var feedItems = new List<RssFeedItem>();
            
            try
            {
                var response = await _httpClient.GetStringAsync(url);
                using (var reader = XmlReader.Create(new StringReader(response)))
                {
                    var feed = SyndicationFeed.Load(reader);
                    
                    foreach (var item in feed.Items)
                    {
                        var feedItem = new RssFeedItem
                        {
                            Id = Guid.NewGuid().ToString(),
                            Title = item.Title?.Text ?? "",
                            Description = item.Summary?.Text ?? "",
                            Link = item.Links.FirstOrDefault()?.Uri.ToString() ?? "",
                            PublishDate = item.PublishDate.UtcDateTime,
                            Author = item.Authors.FirstOrDefault()?.Name ?? ""
                        };
                        
                        // Find torrent link
                        foreach (var link in item.Links)
                        {
                            if (link.MediaType?.Contains("torrent") == true || 
                                link.Uri.ToString().EndsWith(".torrent") ||
                                link.Uri.ToString().StartsWith("magnet:"))
                            {
                                feedItem.TorrentUrl = link.Uri.ToString();
                                break;
                            }
                        }
                        
                        // If no torrent link found, use the main link
                        if (string.IsNullOrEmpty(feedItem.TorrentUrl))
                        {
                            feedItem.TorrentUrl = feedItem.Link;
                        }
                        
                        feedItems.Add(feedItem);
                    }
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, $"Error fetching feed: {url}");
                throw;
            }
            
            return feedItems;
        }
        
        private void ApplyRulesToItem(RssFeed feed, RssFeedItem item)
        {
            item.IsMatched = false;
            item.MatchedRule = string.Empty;
            
            // Apply simple string rules
            foreach (var rulePattern in feed.Rules)
            {
                if (item.Title.Contains(rulePattern, StringComparison.OrdinalIgnoreCase))
                {
                    item.IsMatched = true;
                    item.MatchedRule = rulePattern;
                    item.Category = feed.DefaultCategory;
                    break;
                }
            }
            
            // Apply advanced rules
            foreach (var rule in feed.AdvancedRules.Where(r => r.IsEnabled).OrderByDescending(r => r.Priority))
            {
                bool isMatch = false;
                
                if (rule.IsRegex)
                {
                    try
                    {
                        var regex = new Regex(rule.Pattern, 
                            rule.IsCaseSensitive ? RegexOptions.None : RegexOptions.IgnoreCase);
                        isMatch = regex.IsMatch(item.Title);
                    }
                    catch (Exception ex)
                    {
                        _logger.LogError(ex, $"Invalid regex pattern: {rule.Pattern}");
                    }
                }
                else
                {
                    var comparison = rule.IsCaseSensitive ? 
                        StringComparison.Ordinal : StringComparison.OrdinalIgnoreCase;
                    isMatch = item.Title.Contains(rule.Pattern, comparison);
                }
                
                if (isMatch)
                {
                    item.IsMatched = true;
                    item.MatchedRule = rule.Name;
                    item.Category = rule.Category;
                    break;
                }
            }
        }
        
        private async Task DownloadMatchedItemAsync(RssFeedItem item)
        {
            try
            {
                var config = _configService.GetConfiguration();
                var downloadDir = config.DownloadDirectory;
                
                if (string.IsNullOrEmpty(downloadDir))
                {
                    _logger.LogWarning("Download directory not configured");
                    return;
                }
                
                _logger.LogInformation($"Downloading matched item: {item.Title}");
                
                // Add torrent to Transmission
                int torrentId = await _transmissionClient.AddTorrentAsync(item.TorrentUrl, downloadDir);
                
                // Update feed item
                item.IsDownloaded = true;
                item.DownloadDate = DateTime.UtcNow;
                item.TorrentId = torrentId;
                
                _logger.LogInformation($"Added torrent: {item.Title} (ID: {torrentId})");
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, $"Error downloading item: {item.Title}");
                item.RejectionReason = ex.Message;
            }
        }
        
        private void CleanupOldItems(RssFeed feed)
        {
            if (feed.MaxHistoryItems <= 0)
                return;
            
            var feedItems = _feedItems.Where(i => i.FeedId == feed.Id).ToList();
            
            if (feedItems.Count > feed.MaxHistoryItems)
            {
                // Keep all downloaded items
                var downloadedItems = feedItems.Where(i => i.IsDownloaded).ToList();
                
                // Keep most recent non-downloaded items up to the limit
                var nonDownloadedItems = feedItems.Where(i => !i.IsDownloaded)
                    .OrderByDescending(i => i.PublishDate)
                    .Take(feed.MaxHistoryItems - downloadedItems.Count)
                    .ToList();
                
                // Set new list
                var itemsToKeep = downloadedItems.Union(nonDownloadedItems).ToList();
                _feedItems.RemoveAll(i => i.FeedId == feed.Id && !itemsToKeep.Contains(i));
            }
        }
        
        public async Task MarkItemAsDownloadedAsync(string itemId)
        {
            var item = _feedItems.FirstOrDefault(i => i.Id == itemId);
            if (item != null)
            {
                item.IsDownloaded = true;
                item.DownloadDate = DateTime.UtcNow;
            }
        }
    }

    public class RssFeedBackgroundService : BackgroundService
    {
        private readonly ILogger<RssFeedBackgroundService> _logger;
        private readonly IServiceProvider _serviceProvider;

        public RssFeedBackgroundService(
            ILogger<RssFeedBackgroundService> logger,
            IServiceProvider serviceProvider)
        {
            _logger = logger;
            _serviceProvider = serviceProvider;
        }

        protected override async Task ExecuteAsync(CancellationToken stoppingToken)
        {
            _logger.LogInformation("RSS feed background service started");
            
            while (!stoppingToken.IsCancellationRequested)
            {
                try
                {
                    using (var scope = _serviceProvider.CreateScope())
                    {
                        var rssFeedManager = scope.ServiceProvider.GetRequiredService<IRssFeedManager>();
                        var configService = scope.ServiceProvider.GetRequiredService<IConfigService>();
                        
                        await rssFeedManager.RefreshFeedsAsync(stoppingToken);
                        
                        var config = configService.GetConfiguration();
                        var interval = TimeSpan.FromMinutes(config.CheckIntervalMinutes);
                        
                        _logger.LogInformation($"Next refresh in {interval.TotalMinutes} minutes");
                        await Task.Delay(interval, stoppingToken);
                    }
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Error refreshing RSS feeds");
                    await Task.Delay(TimeSpan.FromMinutes(5), stoppingToken);
                }
            }
        }
    }
}