using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Logging;
using Cronos;
using TransmissionRssManager.Core;

namespace TransmissionRssManager.Services
{
    public interface ISchedulerService
    {
        DateTime GetNextScheduledRun(string cronExpression);
        bool IsValidCronExpression(string cronExpression);
        Task ScheduleFeedRefreshAsync(RssFeed feed, CancellationToken cancellationToken);
        Task ScheduleAllFeedsAsync(CancellationToken cancellationToken);
    }
    
    public class SchedulerService : ISchedulerService
    {
        private readonly ILogger<SchedulerService> _logger;
        private readonly IServiceProvider _serviceProvider;
        private readonly ILoggingService _loggingService;
        
        public SchedulerService(
            ILogger<SchedulerService> logger,
            IServiceProvider serviceProvider,
            ILoggingService loggingService)
        {
            _logger = logger;
            _serviceProvider = serviceProvider;
            _loggingService = loggingService;
        }
        
        public DateTime GetNextScheduledRun(string cronExpression)
        {
            try
            {
                var expression = CronExpression.Parse(cronExpression);
                var nextRun = expression.GetNextOccurrence(DateTime.UtcNow);
                return nextRun ?? DateTime.UtcNow.AddMinutes(30); // Default to 30 minutes if parsing fails
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, $"Error parsing cron expression: {cronExpression}");
                return DateTime.UtcNow.AddMinutes(30); // Default to 30 minutes
            }
        }
        
        public bool IsValidCronExpression(string cronExpression)
        {
            try
            {
                CronExpression.Parse(cronExpression);
                return true;
            }
            catch
            {
                return false;
            }
        }
        
        public async Task ScheduleFeedRefreshAsync(RssFeed feed, CancellationToken cancellationToken)
        {
            if (!feed.Enabled)
            {
                _logger.LogInformation($"Feed '{feed.Name}' is disabled, skipping schedule");
                return;
            }
            
            string cronExpression = feed.Schedule;
            
            if (string.IsNullOrEmpty(cronExpression) || !IsValidCronExpression(cronExpression))
            {
                _logger.LogWarning($"Invalid cron expression for feed: {feed.Name}, using default schedule");
                cronExpression = "*/30 * * * *"; // Default is every 30 minutes
            }
            
            var nextRun = GetNextScheduledRun(cronExpression);
            var delay = nextRun - DateTime.UtcNow;
            
            if (delay.TotalMilliseconds <= 0)
            {
                delay = TimeSpan.FromSeconds(5); // If next run is in the past, schedule for 5 seconds from now
            }
            
            _logger.LogInformation($"Scheduling feed '{feed.Name}' to run at {nextRun} (in {delay.TotalMinutes:F1} minutes)");
            
            try
            {
                await Task.Delay(delay, cancellationToken);
                
                if (cancellationToken.IsCancellationRequested)
                {
                    return;
                }
                
                // Process the feed
                using (var scope = _serviceProvider.CreateScope())
                {
                    var rssFeedManager = scope.ServiceProvider.GetRequiredService<IRssFeedManager>();
                    
                    _loggingService.Log(
                        LogLevel.Information,
                        $"Running scheduled refresh for feed: {feed.Name}",
                        "SchedulerService",
                        new Dictionary<string, string> { { "FeedId", feed.Id }, { "FeedName", feed.Name } }
                    );
                    
                    try
                    {
                        // We need to get the latest feed configuration because it might have changed
                        var feeds = await rssFeedManager.GetFeedsAsync();
                        var updatedFeed = feeds.FirstOrDefault(f => f.Id == feed.Id);
                        
                        if (updatedFeed != null && updatedFeed.Enabled)
                        {
                            await rssFeedManager.RefreshFeedAsync(updatedFeed.Id, cancellationToken);
                            
                            _loggingService.Log(
                                LogLevel.Information,
                                $"Successfully refreshed feed: {updatedFeed.Name}",
                                "SchedulerService",
                                new Dictionary<string, string> { { "FeedId", updatedFeed.Id }, { "FeedName", updatedFeed.Name } }
                            );
                        }
                        else
                        {
                            _logger.LogInformation($"Feed '{feed.Name}' no longer exists or is disabled, skipping refresh");
                        }
                    }
                    catch (Exception ex)
                    {
                        _loggingService.Log(
                            LogLevel.Error,
                            $"Error refreshing feed: {feed.Name}. {ex.Message}",
                            "SchedulerService",
                            new Dictionary<string, string> { 
                                { "FeedId", feed.Id }, 
                                { "FeedName", feed.Name },
                                { "ErrorMessage", ex.Message },
                                { "StackTrace", ex.StackTrace }
                            }
                        );
                    }
                }
                
                // Reschedule the feed for its next run
                if (!cancellationToken.IsCancellationRequested)
                {
                    // Get the latest feed configuration again
                    using (var scope = _serviceProvider.CreateScope())
                    {
                        var rssFeedManager = scope.ServiceProvider.GetRequiredService<IRssFeedManager>();
                        var feeds = await rssFeedManager.GetFeedsAsync();
                        var updatedFeed = feeds.FirstOrDefault(f => f.Id == feed.Id);
                        
                        if (updatedFeed != null)
                        {
                            // Schedule the feed with its potentially updated configuration
                            await ScheduleFeedRefreshAsync(updatedFeed, cancellationToken);
                        }
                    }
                }
            }
            catch (OperationCanceledException)
            {
                // Task was canceled, just exit
                _logger.LogInformation($"Scheduled feed refresh for '{feed.Name}' was canceled");
            }
            catch (Exception ex)
            {
                _loggingService.Log(
                    LogLevel.Error,
                    $"Error in scheduler for feed: {feed.Name}. {ex.Message}",
                    "SchedulerService",
                    new Dictionary<string, string> { 
                        { "FeedId", feed.Id }, 
                        { "FeedName", feed.Name },
                        { "ErrorMessage", ex.Message },
                        { "StackTrace", ex.StackTrace }
                    }
                );
                
                // Reschedule after error with a delay
                if (!cancellationToken.IsCancellationRequested)
                {
                    await Task.Delay(TimeSpan.FromMinutes(5), cancellationToken);
                    await ScheduleFeedRefreshAsync(feed, cancellationToken);
                }
            }
        }
        
        public async Task ScheduleAllFeedsAsync(CancellationToken cancellationToken)
        {
            using (var scope = _serviceProvider.CreateScope())
            {
                var rssFeedManager = scope.ServiceProvider.GetRequiredService<IRssFeedManager>();
                var feeds = await rssFeedManager.GetFeedsAsync();
                
                var tasks = feeds
                    .Where(f => f.Enabled)
                    .Select(feed => ScheduleFeedRefreshAsync(feed, cancellationToken))
                    .ToList();
                
                _logger.LogInformation($"Scheduled {tasks.Count} feeds for refresh");
            }
        }
    }
    
    public class FeedSchedulerBackgroundService : BackgroundService
    {
        private readonly ILogger<FeedSchedulerBackgroundService> _logger;
        private readonly IServiceProvider _serviceProvider;
        private readonly ISchedulerService _schedulerService;
        
        public FeedSchedulerBackgroundService(
            ILogger<FeedSchedulerBackgroundService> logger,
            IServiceProvider serviceProvider,
            ISchedulerService schedulerService)
        {
            _logger = logger;
            _serviceProvider = serviceProvider;
            _schedulerService = schedulerService;
        }
        
        protected override async Task ExecuteAsync(CancellationToken stoppingToken)
        {
            _logger.LogInformation("Feed scheduler background service started");
            
            // Initial delay to let other services initialize
            await Task.Delay(TimeSpan.FromSeconds(10), stoppingToken);
            
            while (!stoppingToken.IsCancellationRequested)
            {
                try
                {
                    await _schedulerService.ScheduleAllFeedsAsync(stoppingToken);
                    
                    // Check for new feeds or schedule changes every 5 minutes
                    await Task.Delay(TimeSpan.FromMinutes(5), stoppingToken);
                }
                catch (OperationCanceledException)
                {
                    // Service is shutting down
                    break;
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Error in feed scheduler background service");
                    await Task.Delay(TimeSpan.FromMinutes(1), stoppingToken);
                }
            }
            
            _logger.LogInformation("Feed scheduler background service stopped");
        }
    }
}